#include <mpi.h>
#include <stdio.h>
#include <math.h>

#define ALL MPI_COMM_WORLD

double sqrt();

void main(int argc, char **argv) {

  /* Integer, Double type variable declaration */
  double pisquared, sum, h, x, y;
  /* Approximated original PI value */
  double PI25DT = 3.14159265358979323842643;
  int id, size, i, j, grid_rank;
  /* Fixed number of Intervals */
  int n = 10000;     
  int dim[2], wrap[2], coord[2];
  MPI_Comm grid;
  
  /* Initialize MPI functions */
  MPI_Init(&argc, &argv);

  /* In order for a process to find out its rank */
  MPI_Comm_rank(ALL, &id);

  /* Number of Dimensions on row and column */ 
  dim[0] = 2;
  dim[1] = 4;
  wrap[0] = wrap[1] = 0;

  /* Creates a new communicator by catching a Cartesian topology */
  MPI_Cart_create(ALL, 2, dim, wrap, 0, &grid);

  MPI_Comm_rank(grid, &grid_rank);

  /* Return the coordinates of the process with rank */
  MPI_Cart_coords(grid, grid_rank, 2, coord);

  /* From now on it is the formula to get the approx. value of square(pi) */
  sum = 0;
  h = 1/(double)n;

  for (i=coord[0]; i<n; i+=dim[0]) {
    for (j=coord[1]; j<n; j+=dim[1])  {

      x = (double)i * h;
      y = (double)j * h;

      sum += h*h*(1/((1+(x+.5*h)*(x+.5*h))*(1+(y+.5*h)*(y+.5*h))));
    }
  }
  
  sum *= 16;
  printf("sum is:%f\n", sum);
  MPI_Reduce(&sum, &pisquared, 1, MPI_DOUBLE, MPI_SUM, 0, ALL);
  if (id == 0)
    {
    printf("pi square is:%.16f, and error is:%.16f\n",
            pisquared, fabs(pisquared - PI25DT * PI25DT));
    printf("pi is:%.16f\n", sqrt(pisquared));
    }
  
  MPI_Finalize();
}
